@@ -64,8 +64,8 @@ get_steam_code(unsigned const char *hmac)
     char code[6];
     size_t steam_alphabet_len = strlen(steam_alphabet);
     for (int i = 0; i < 5; i++) {
-        int mod = bin_code % steam_alphabet_len;
-        bin_code = bin_code / steam_alphabet_len;
+        int mod = (int)(bin_code % steam_alphabet_len);
+        bin_code = (int)(bin_code / steam_alphabet_len);
         code[i] = steam_alphabet[mod];
     }
     code[5] = '\0';
@@ -78,25 +78,12 @@ static int
 truncate(unsigned const char *hmac, int digits_length, int algo)
 {
     // take the lower four bits of the last byte
-    int offset = 0;
-    switch (algo) {
-        case SHA1:
-            offset = (hmac[SHA1_DIGEST_SIZE-1] & 0x0f);
-            break;
-        case SHA256:
-            offset = (hmac[SHA256_DIGEST_SIZE-1] & 0x0f);
-            break;
-        case SHA512:
-            offset = (hmac[SHA512_DIGEST_SIZE-1] & 0x0f);
-            break;
-        default:
-            break;
-    }
+    int offset = hmac[gcry_md_get_algo_dlen (algo) - 1] & 0x0f;
 
     // Starting from the offset, take the successive 4 bytes while stripping the topmost bit to prevent it being handled as a signed integer
     int bin_code = ((hmac[offset] & 0x7f) << 24) | ((hmac[offset + 1] & 0xff) << 16) | ((hmac[offset + 2] & 0xff) << 8) | ((hmac[offset + 3] & 0xff));
 
-    int token = bin_code % DIGITS_POWER[digits_length];
+    int token = (int)(bin_code % DIGITS_POWER[digits_length]);
 
     return token;
 }
@@ -106,7 +93,7 @@ static unsigned char *
 compute_hmac(const char *K, long C, int algo)
 {
     baseencode_error_t err;
-    size_t secret_len = (size_t) ((strlen(K) + 1.6 - 1) / 1.6);
+    size_t secret_len = (size_t)((strlen(K) + 1.6 - 1) / 1.6);
 
     char *normalized_K = normalize_secret (K);
     if (normalized_K == NULL) {
@@ -124,13 +111,39 @@ compute_hmac(const char *K, long C, int algo)
         C_reverse_byte_order[i] = ((unsigned char *) &C)[j];
 
     gcry_md_hd_t hd;
-    gcry_md_open(&hd, algo, GCRY_MD_FLAG_HMAC);
-    gcry_md_setkey(hd, secret, secret_len);
-    gcry_md_write(hd, C_reverse_byte_order, sizeof(C_reverse_byte_order));
+    gpg_error_t gpg_err = gcry_md_open (&hd, algo, GCRY_MD_FLAG_HMAC);
+    if (gpg_err) {
+        printf("%s\n", "Error while opening the cipher handle.");
+        return NULL;
+    }
+    gpg_err = gcry_md_setkey (hd, secret, secret_len);
+    if (gpg_err) {
+        printf("%s\n", "Error while setting the cipher key.");
+        gcry_md_close (hd);
+        return NULL;
+    }
+    gcry_md_write (hd, C_reverse_byte_order, sizeof(C_reverse_byte_order));
     gcry_md_final (hd);
-    unsigned char *hmac = gcry_md_read(hd, algo);
 
-    free(secret);
+    unsigned char * hmac_tmp = gcry_md_read (hd, algo);
+    if (hmac_tmp == NULL) {
+        fprintf(stderr, "Error getting digest\n");
+        gcry_md_close (hd);
+        return NULL;
+    }
+
+    size_t dlen = gcry_md_get_algo_dlen(algo);
+    unsigned char *hmac = malloc (dlen);
+    if (hmac == NULL) {
+        perror("Error allocating memory");
+        gcry_md_close (hd);
+        return NULL;
+    }
+    memcpy (hmac, hmac_tmp, dlen);
+
+    free (secret);
+
+    gcry_md_close (hd);
 
     return hmac;
 }
@@ -215,8 +228,11 @@ get_hotp(const char *secret, long timestamp, int digits, int algo, cotp_error_t
         *err_code = INVALID_B32_INPUT;
         return NULL;
     }
+
     int tk = truncate(hmac, digits, algo);
     char *token = finalize(digits, tk);
+
+    free(hmac);
     return token;
 }
 
@@ -289,7 +305,10 @@ get_steam_totp_at (const char *secret, long current_timestamp, int period, cotp_
         return NULL;
     }
 
-    return get_steam_code(hmac);
+    char * totp = get_steam_code(hmac);
+
+    free(hmac);
+    return totp;
 }
 
 
